/* exs_private.h */

#ifndef EXS_PRIVATE_H
#define EXS_PRIVATE_H

/*
 * Copyright 2007-2016 University of New Hampshire Interoperability Laboratory
 * All rights reserved.
 *
 * This file is part of the UNH EXS implementation of the
 * Consortium Specification: Extended Sockets API (ES-API) Issue 1.0,
 * Document Number C050 published by The Open Group, January 2005,
 * and copyright 2005 by The Open Group.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING.GPL in the main directory of this source tree, or the
 * BSD license, available from the file COPYING.BSD in the main directory
 * of this source tree.
 *
 * This material is based upon work supported by the National Science
 * Foundation under Grants No. OCI-1127228 and DGE-0913620.
 *
 * Any opinions, findings, and conclusions or recommendations
 * expressed in this material are those of the authors and do not
 * necessarily reflect the views of the National Science Foundation.
 */

/*
 * The UNH IOL implementation of the Extended Sockets API (ES-API) Issue 1.0
 *
 * defined in:	The Interconnect Software Consortium Issue 1.0
 *		ISBN: 1-931624-52-6
 *		Published by The open Group, January 2005.
 *
 * all page references in this source code are to pages in that document.
 */

/*
 * Most of the coding in this file done by:
 *	Bob Russell
 *	rdr@iol.unh.edu
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#ifdef HAVE_VALGRIND_MEMCHECK_H
#include <valgrind/memcheck.h>
#else
#define VALGRIND_MAKE_MEM_DEFINED(x,y)
#define VALGRIND_MAKE_MEM_UNDEFINED(x,y)
#endif

#define EXS_EXPORT __attribute__((visibility("default")))
#define EXS_WARN_UNUSED_RETURN __attribute__((warn_unused_result))

#define MIN(x,y) (((x) <= (y)) ? (x) : (y))

#include <trace.h>

#if defined(ALLOW_EXS_STATISTICS)

	enum statistics_counter_type {
		completion_loops,
		good_ibv_poll_cqs,
		empty_ibv_poll_cqs,
		good_ibv_get_cq_events,
		good_completions,
		max_ibv_poll_cq_completions,
		send_ack_completions,
		send_advert_completions,
		send_rdma_write_completions,
		recv_advert_completions,
		recv_rdma_with_imm_completions,
		recv_rdma_with_imm_bytes,
		recv_eof_completions,
		total_setup_completions,
		matched_recv_await,
		unmatched_recv_await,
		send_done_after_sending,
		send_done_after_completion,
		total_setup_sends,
		advert_blocks_sent,
		signaled_advert_blocks_sent,
		unsignaled_advert_blocks_sent,
		signaled_rdma_write_blocks_sent,
		unsignaled_rdma_write_blocks_sent,
		rdma_write_blocks_sent,
		rdma_write_bytes_sent,
		ack_blocks_sent,
		recv_ack_completions,
		recv_ack_buf_completions,
		eof_blocks_sent,
		no_tblock_on_send_ack,
		extra_advert_blocks,
		direct_rdma_write_blocks_sent,
		indirect_rdma_write_blocks_sent,
		no_credits_on_send_wwi,
		max_used_wwi_credits,
		no_credits_on_send_ack,
		max_used_ack_credits,
		no_credits_on_send_advert,
		max_used_advert_credits,
		direct_indirect_switches,
		ibv_reg_mr_calls,
		ibv_dereg_mr_calls,
		max_recv_buf_n_full,
		max_remote_recv_buf_n_full,
		recv_buf_full_on_send,
		recv_buf_empty_on_recv,
		max_statistics_counter_type
	};

#define STATISTICS_INCREMENT(arg)	\
	do	{		\
		statistics_counters[arg]++;		\
	} while (0)

#define STATISTICS_DECREMENT(arg)	\
	do	{		\
		statistics_counters[arg]--;		\
	} while (0)

#define STATISTICS_ADD(arg, val)	\
	do	{		\
		statistics_counters[arg] += val;	\
	} while (0)

/* No STATISTICS_SET_MIN since values are initialized to 0 */

#define STATISTICS_SET_MAX(arg, val)			\
	do	{					\
		unsigned long v__ = (val);		\
		if (v__ > statistics_counters[arg]) {	\
			statistics_counters[arg] = v__;	\
		}					\
	} while (0)

#else

#define STATISTICS_INCREMENT(arg)	\
	do	{		\
	} while (0)

#define STATISTICS_ADD(arg, val)	\
	do	{		\
	} while (0)

#define STATISTICS_SET_MAX(arg, val)	\
	do	{		\
	} while (0)

#endif

#endif /* EXS_PRIVATE_H */
