/*
 * Copyright 2007-2017 University of New Hampshire Interoperability Laboratory
 * All rights reserved.
 *
 * This file is part of the UNH EXS implementation of the
 * Consortium Specification: Extended Sockets API (ES-API) Issue 1.0,
 * Document Number C050 published by The Open Group, January 2005,
 * and copyright 2005 by The Open Group.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING.GPL in the main directory of this source tree, or the
 * BSD license, available from the file COPYING.BSD in the main directory
 * of this source tree.
 *
 * This material is based upon work supported by the National Science
 * Foundation under Grants No. OCI-1127228 and DGE-0913620.
 *
 * Any opinions, findings, and conclusions or recommendations
 * expressed in this material are those of the authors and do not
 * necessarily reflect the views of the National Science Foundation.
 */

#ifndef _PARAM_H
#define _PARAM_H

/* simple linked list node */
struct node {
	int value;
	struct node *next;
};

/* param struct - used to hold parameters for testing */
typedef struct {
	/* name of current function */
	char *func;

	/* blocking (0) or non-blocking (1) flag */
	int mode;

	int list;		//list flag
	char *list_opt;		//list option

	int timeout;		//SIGALARM timer

	int version;
	int fd;
	int family;
	int socktype;
	int protocol;

	int client_fd;
	int listener_fd;

	const struct sockaddr *address;
	socklen_t address_len;

	int recv_fd;

	char *addr;		/* address */
	char *port;		/* port */

	char *test_name;
	char *test_value;
	int test_number;

	/* current test iteration */
	struct node *iter;

	/* buffer for sending data */
	int SEND_BUFSIZE;
	char *send_buf;

	/* buffer for receiving data */
	int RECV_BUFSIZE;
	char *recv_buf;

	/* exs specific parameters */

	int flags;
	int close_flags;

	struct timeval *to;

	exs_qhandle_t qh;

	exs_ahandle_t ah;

	exs_mhandle_t mh;

	exs_mhandle_t send_mh;
	exs_mhandle_t recv_mh;

	exs_event_t dequeue_event;

	exs_event_t *evtvec;
	int evtvec_cnt;

	int qattr_wait;

	exs_xferfile_t *sendvec;
	int sendvec_cnt;

	int file_length;

} param;

#endif
